import { jsx, Fragment, jsxs } from "react/jsx-runtime";
import { createPortal, useThree, useFrame } from "@react-three/fiber";
import { memo, useState, useRef, useLayoutEffect } from "react";
import { Scene } from "three";
import { syncCamera } from "./main11.mjs";
import { useCoordsToMatrix } from "./main12.mjs";
import { useR3M } from "./main13.mjs";
const Coordinates = memo(({
  latitude,
  longitude,
  altitude = 0,
  children
}) => {
  const [scene] = useState(() => new Scene());
  const r3m = useR3M();
  const origin = useCoordsToMatrix({
    latitude,
    longitude,
    altitude,
    fromLngLat: r3m.fromLngLat
  });
  return /* @__PURE__ */ jsx(Fragment, { children: createPortal(/* @__PURE__ */ jsxs(Fragment, { children: [
    /* @__PURE__ */ jsx(RenderAtCoords, { r3m, origin }),
    children
  ] }), scene, { events: { priority: 2 } }) });
});
Coordinates.displayName = "Coordinates";
function RenderAtCoords({ r3m, origin }) {
  const { gl, scene, set } = useThree();
  const cameraRef = useRef(null);
  useFrame(() => {
    if (!cameraRef.current)
      return;
    syncCamera(cameraRef.current, origin, r3m.viewProjMx);
    gl.render(scene, cameraRef.current);
  });
  useLayoutEffect(() => {
    if (!cameraRef.current)
      return;
    set({
      invalidate: () => {
        if (!r3m.map)
          return;
        r3m.map.triggerRepaint();
      },
      camera: cameraRef.current
    });
  }, [set, r3m]);
  return /* @__PURE__ */ jsx("perspectiveCamera", { ref: cameraRef });
}
export {
  Coordinates
};
//# sourceMappingURL=main4.mjs.map
