import { MathUtils } from "three";
import { earthRadius } from "./main14.mjs";
const mercatorScaleLookup = {};
function getMercatorScale(lat) {
  const index = Math.round(lat * 1e3);
  if (mercatorScaleLookup[index] === void 0) {
    mercatorScaleLookup[index] = 1 / Math.cos(lat * MathUtils.DEG2RAD);
  }
  return mercatorScaleLookup[index];
}
function averageMercatorScale(originLat, pointLat, steps = 10) {
  let totalScale = 0;
  const latStep = (pointLat - originLat) / steps;
  for (let i = 0; i <= steps; i++) {
    const lat = originLat + latStep * i;
    totalScale += getMercatorScale(lat);
  }
  return totalScale / (steps + 1);
}
function coordsToVector3(point, origin) {
  const latitudeDiff = (point.latitude - origin.latitude) * MathUtils.DEG2RAD;
  const longitudeDiff = (point.longitude - origin.longitude) * MathUtils.DEG2RAD;
  const altitudeDiff = (point.altitude || 0) - (origin.altitude || 0);
  const x = longitudeDiff * earthRadius * Math.cos(origin.latitude * MathUtils.DEG2RAD);
  const y = altitudeDiff;
  const steps = Math.ceil(Math.abs(point.latitude - origin.latitude)) * 100 + 1;
  const avgScale = averageMercatorScale(origin.latitude, point.latitude, steps);
  const z = -latitudeDiff * earthRadius / getMercatorScale(origin.latitude) * avgScale;
  return [x, y, z];
}
export {
  averageMercatorScale,
  coordsToVector3
};
//# sourceMappingURL=main5.mjs.map
